"""
Train
=====
Defines functions which train models and write model artifacts to disk.
"""
from __future__ import print_function

import os
import tempfile

import tensorflow as tf
from tensorflow.examples.tutorials.mnist import input_data

from tensorflow_mnist import model, paths


def train(path):
    """
    Train a decision tree classifier using a floating point feature matrix and
    a categorical classification target.

    Arguments:
        path (str): The path indicating where to save the final model artifacts
    """

    # Construct the model graph
    graph, x, y, step, initializer, accuracy, prediction = model.build()

    # Start a training session
    with tf.Session(graph=graph) as sess:

        # Initialize the graph
        sess.run(initializer)

        # Train the model for 1000 steps
        mnist = input_data.read_data_sets(tempfile.mkdtemp(), one_hot=True)
        for _ in range(1000):
            batch_xs, batch_ys = mnist.train.next_batch(100)
            sess.run(step, feed_dict={x: batch_xs, y: batch_ys})

        # Display accuracy measurement
        print(sess.run(accuracy, feed_dict={x: mnist.test.images,
                                            y: mnist.test.labels}))
        # Save the variable data to disk
        os.makedirs(path)
        saver = tf.train.Saver()
        saver.save(sess, path)

    print('Success!')


def main():
    """
    Load features and labels, train the neural network, and serialize model
    artifact.

    Note: This is the training entrypoint used by baklava!
    """
    path = paths.model('mnist')
    train(path)



from __future__ import print_function

import logging
import functools


logger = logging.getLogger('baklava')


@functools.lru_cache()
def load_model():
    """
    Example initializer which contains logging

    Returns:
        result (int): A constant value to use a the model
    """
    logger.info('load_model called')
    return 1


def hosted_function(payload):
    """
    Example hosted function which always returns a 201 with the message "OK".
    This uses a tuple return type containing the status code which is directly
    supported by Flask.

    See: https://flask.palletsprojects.com/en/1.1.x/quickstart/#about-responses

    Arguments:
        payload (dict[str, object]): This is the payload that will eventually
            be sent to the SageMaker server using a POST request to the
            `invocations` route.

    Returns:
        response (tuple[str, int]):
            message (str): The response message.
            status (int): The status code.
    """
    logger.info('hosted_function called')
    return 'OK', 201




import boto3
client = boto3.Session().client('sagemaker')

response = client.create_endpoint(
    EndpointName='stumblebum-lenitives-endpoint',
    EndpointConfigName='stumblebum-lenitives-endpoint',
    Tags=[
        {
            'Key': 'mlctl',
            'Value': 'asdf'
        },
    ]
)

print(response)


from setuptools import setup, find_packages

setup(
    name='sklearn_tree',
    version='0.0.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'sklearn',
        'pandas',
        'scipy',
    ],
    dockerlines=[
        'COPY /mlctlsriracha-0.0.11.tar.gz /opt/mlctlsriracha-0.0.11.tar.gz',
        'RUN pip install /opt/mlctlsriracha-0.0.11.tar.gz'
    ],
    python_version='3.6',
    entry_points={
        'baklava.train': [
            'my_training_entrypoint = sklearn_tree.train:main',
        ],
        'baklava.predict': [
            'my_prediction_entrypoint = sklearn_tree.predict:main',
        ]
    }
)



import gevent.monkey
gevent.monkey.patch_all()

import os
import json
import logging
import sys
import inspect
import signal
import subprocess

import six
import psutil
from flask import Flask, request
import gunicorn.app.base

from mlsriracha.deploy import DeployAdapter

# ------------------------------------------------------------------------------
# Logging
# ------------------------------------------------------------------------------

logger = logging.getLogger('mlbaklava')
logger.addHandler(logging.NullHandler())


class FlaskRequestFormatter(logging.Formatter):
    """
    Class to add tid to log messages.
    """

    def format(self, record):
        # Runtime error will occur if not in a request context
        try:
            record.tid = request.headers.get("X-Amzn-SageMaker-Custom-Attributes", "-")
        except RuntimeError as e:
            record.tid = "-"
        return super().format(record)


def configure_logger():
    """
    Configure a logger to use standard format and levels.
    """
    handler = logging.StreamHandler(stream=sys.stdout)
    formatter = FlaskRequestFormatter(
        fmt="time=%(asctime)s.%(msecs)03d level=%(levelname)s tid=%(tid)s file=%(filename)s:%(lineno)s %(message)s",
        datefmt='%Y-%m-%d,%H:%M:%S'
    )
    handler.setFormatter(formatter)
    handler.setLevel(logging.DEBUG)
    logger.addHandler(handler)
    logger.setLevel(logging.DEBUG)


# ------------------------------------------------------------------------------
# Server Setup
# ------------------------------------------------------------------------------

def application(func):

    configure_logger()

    # --------------------------------------------------------------------------
    # Check that function is lambda-like
    # --------------------------------------------------------------------------

    if sys.version_info.major == 2:
        spec = inspect.getargspec(func)
    else:
        spec = inspect.getfullargspec(func)

    if len(spec.args) != 2:
        raise ValueError(
            'Expected method to have exactly 1 argument, got {} arguments: {}'
            .format(len(spec.args), spec.args)
        )

    if spec.varargs is not None:
        raise ValueError(
            'Expected method to have 1 argument, but got variable arguments: {}'
            .format(spec.varargs)
        )

    # --------------------------------------------------------------------------
    # Flask
    # --------------------------------------------------------------------------

    app = Flask(__name__)

    # --------------------------------------------------------------------------
    # Routes
    # --------------------------------------------------------------------------

    @app.route('/invocations', methods=['POST'])
    def invocations():
        """
        Execute a python function
        Uses the JSON body of the POST request to as an input to the python
        function. The JSON inputwill be parsed into a dictionary and passed into
        the function. The output of the function is expected to be a JSON
        string.
        ---
        tags:
            - endpoint
        consumes:
            - application/json
        produces:
            - application/json
        parameters:

            - in: body
              name: body
              description: The data to use as the model inputs
              required: true
              type: object
              schema:
                properties:
                  tid:
                    type: str
                    default: "748babba-d287-4082-9c12-ea46e7a5146e"
                  id:
                    type: str
                    default: "123456789"
                  data:
                    type: object
                    schema:
                      properties:
                        example:
                          type: int
                          default: 1
        """

        if request.is_json:
            data = request.get_json()  # type: dict[str, object]
        else:
            data = request.get_data()  # type: bytes

        # Create prediction adapter
        da = DeployAdapter(os.getenv('sriracha_provider'))

        # Execute lambda
        try:
            result = func(da, data)
        except TypeError:
            # print('User provided function does not support PredictAdapter')
            result = func(data)
        return result

    @app.route('/ping', methods=['GET'])
    def ping():
        """
        Check if the server is running

        Will always output success as long as the server is running. Otherwise
        expect a 404.
        ---
        tags:
            - endpoint
        produces:
            - application/json
        """
        return json.dumps({'success': True})

    def sigterm_handler(signum, frame):
        # Shutdown Flask application when SIGTERM is received as a result of
        # "docker stop" command
        app.shutdown()

    signal.signal(signal.SIGTERM, sigterm_handler)
    return app


class Server(gunicorn.app.base.BaseApplication):

    def __init__(self, func, initializer=None, **options):
        self.initializer = initializer
        self.options = options or {}
        self.app = application(func)
        super(Server, self).__init__()

    def load_config(self):
        config = dict([(key, value) for key, value in six.iteritems(self.options)
                       if key in self.cfg.settings and value is not None])
        for key, value in six.iteritems(config):
            self.cfg.set(key.lower(), value)

        # Register initialization function with the worker
        def initialize(arbiter, worker):
            self.initializer()

        if self.initializer:
            self.cfg.set('post_fork', initialize)

    def load(self):
        return self.app


def timeout():
    if 'LAMBDA_SERVER_TIMEOUT' in os.environ:
        return int(os.environ['LAMBDA_SERVER_TIMEOUT'])
    return 600


def number_of_workers():
    """
    Get the number of workers for gunicorn.

    This can be explicitly set by using the LAMBDA_SERVER_WORKERS environment
    variable. The number is expected to be an integer.

    This returns 2 by default. The default option is generally used during
    testing, so use the minimum number of workers to test a multi-worker server.

    By setting the environment variable to "auto", the number of workers will
    scale by the number of physical cores. Note that this can cause unexpected
    behavior in local docker containers. Without specific configuration, docker
    will report 1 physical core (default for OSX).

    Returns:
        n_workers (int): The number of workers
    """
    if 'LAMBDA_SERVER_WORKERS' in os.environ:
        value = os.environ['LAMBDA_SERVER_WORKERS']
        if value == 'auto':
            return psutil.cpu_count(logical=False)
        try:
            return int(value)
        except:
            pass
    return 2


def run(func, initializer=None):

    # Create gunicorn application
    server = Server(
        func=func,
        initializer=initializer,
        bind='unix:/tmp/gunicorn.sock',
        workers=number_of_workers(),
        worker_class='gevent',
        worker_tmp_dir='/dev/shm',
        timeout=timeout(),
    )

    # Start nginx
    subprocess.Popen(['nginx', '-c', '/opt/nginx.conf'])

    # Start gunicorn server
    server.run()


if __name__ == '__main__':
    $entrypoint
    $initializer
    run(entrypoint, initializer)



"""
PyPI
====
Get PyPI indices in a way that is compatible with pip. This uses the same
locations and search precedence that pip uses.
"""
import os
from six.moves import configparser
import sys

import os.path
import appdirs
import warnings
import collections


# ------------------------------------------------------------------------------
# Constants
# ------------------------------------------------------------------------------

config_basename = 'pip.conf'


class kinds:
    USER    = "user"     # User Specific
    GLOBAL  = "global"   # System Wide
    VENV    = "venv"     # Virtual Environment Specific
    ENV     = "env"      # from PIP_CONFIG_FILE
    ENV_VAR = "env-var"  # from Environment Variables


# ------------------------------------------------------------------------------
# Utility Functions
# ------------------------------------------------------------------------------

def running_under_virtualenv():
    """
    Return True if we're running inside a virtualenv, False otherwise.

    """
    if hasattr(sys, 'real_prefix'):
        return True
    elif sys.prefix != getattr(sys, "base_prefix", sys.prefix):
        return True
    return False


def get_legacy_file():
    if os.name == 'nt':
        return os.path.join(os.path.expanduser('~'), 'pip', 'pip.ini')
    return os.path.join(os.path.expanduser('~'), '.pip', 'pip.conf')


def get_site_config_file():
    return os.path.join(appdirs.site_config_dir('pip'), config_basename)


def get_venv_config_file():
    return os.path.join(sys.prefix, config_basename)


def get_new_config_file():
    return os.path.join(appdirs.user_config_dir("pip"), config_basename)


def normalize_name(name):
    # type: (str) -> str
    """Make a name consistent regardless of source (environment or file)
    """
    name = name.strip()
    name = name.lower().replace('_', '-')
    if name.startswith('--'):
        name = name[2:]  # only prefer long opts
    return name


def normalized_keys(section, items):
    # type: (str, Iterable[Tuple[str, Any]]) -> Dict[str, Any]
    """Normalizes items to construct a dictionary with normalized keys.

    This routine is where the names become keys and are made the same
    regardless of source - configuration files or environment.
    """
    normalized = {}
    for name, val in items:
        key = normalize_name(name)
        normalized[key] = val.strip()
    return normalized


def get_environ_vars():
    # type: () -> Iterable[Tuple[str, str]]
    """Returns a generator with all environmental vars with prefix PIP_"""
    for key, val in os.environ.items():
        should_be_yielded = (
                key.startswith("PIP_") and
                key[4:].lower() not in ["version", "help"]
        )
        if should_be_yielded:
            yield key[4:].lower(), val


# ------------------------------------------------------------------------------
# Configuration Loading
# ------------------------------------------------------------------------------

def iter_config_files():
    # type: () -> Iterable[Tuple[Kind, List[str]]]
    """Yields variant and configuration files associated with it.

    This should be treated like items of a dictionary.
    """
    legacy_config_file = get_legacy_file()
    venv_config_file = get_venv_config_file()
    site_config_files = get_site_config_file()
    new_config_file = get_new_config_file()

    # environment variables have the lowest priority
    config_file = os.environ.get('PIP_CONFIG_FILE', None)
    if config_file is not None:
        yield kinds.ENV, [config_file]
    else:
        yield kinds.ENV, []

    # at the base we have any global configuration
    yield kinds.GLOBAL, [site_config_files]

    # The legacy config file is overridden by the new config file
    if not (config_file and os.path.exists(config_file)):
        yield kinds.USER, [legacy_config_file, new_config_file]

    # finally virtualenv configuration first trumping others
    if running_under_virtualenv():
        yield kinds.VENV, [venv_config_file]


def load():
    config_files = dict(iter_config_files())

    override_order = [kinds.GLOBAL, kinds.USER, kinds.VENV, kinds.ENV, kinds.ENV_VAR]

    config = collections.defaultdict(lambda: dict())
    for variant, files in config_files.items():
        for fname in files:

            parser = configparser.RawConfigParser()

            # Read in file
            if os.path.exists(fname):
                try:
                    parser.read(fname)
                except:
                    warnings.warn('Unable to parse config file {}'.format(fname))
                    continue

            for section in parser.sections():
                items = parser.items(section)
                config[variant].update(normalized_keys(section, items))

    config[kinds.ENV_VAR].update(
        normalized_keys(":env:", get_environ_vars())
    )

    retval = {}
    for variant in override_order:
        retval.update(config[variant])
    return retval



"""
Entrypoint
==========
Common operations for handling entrypoint parsing and error checking for both
training and prediction images.
"""


def get(entry, key, name=None):
    """
    Get the entrypoint tuple

    Args:
        entry (dict[str, list[str]): Mapping from types of entrypoints to
            lists of entrypoints for that type.
        key (str): The type of entrypoint to look
            for (i.e. baklava.train)
        name (str): The name of the entrypoint to choose from the list of
            entrypoints. This does not need to be set if only one entrypoint
            exists for the specified section.

    Returns:
        name (str): The name of the entrypoint
        package (str): The package path to access the entrypoint function
        func (str): The name of the function
    """
    assert entry is not None, RuntimeError(
        'To dockerize a distribution, you must define a {} entrypoint'
        .format(key)
    )
    assert key in entry, RuntimeError(
        'To dockerize a using the specified command, you must specify an '
        'entrypoint under the {} key'.format(key)
    )

    assert isinstance(entry[key], list), RuntimeError(
        'Expected a {} of entrypoints in the {} section. Got a {} instead'
        .format(list, key, type(entry[key]))
    )

    assert len(entry[key]) > 0, RuntimeError(
        'No entrypoints specified in the setup.py file. The section {} exists '
        'but there are no entrypoints in the list'.format(key)
    )

    if name is None:
        assert len(entry[key]) == 1, RuntimeError(
            'Multiple entrypoints found for {}. You must specify which one '
            'to use with the entrypoint flag (e.g. python setup.py docker '
            '--entrypoint=main)'.format(key)
        )
        return parse(entry[key][0])

    mapping = mapped(entry[key])

    assert name in mapping, RuntimeError(
        'User specified the {} entrypoint within the {} section but this was '
        'not found. Valid options are: {}'
        .format(name, key, ', '.join(mapping.keys()))
    )

    return mapping[name]


def parse(entrypoint):
    """
    Parse an entrypoint string

    Args:
        entrypoint (str): The entrypoint string to parse.

    Returns:
        name (str): The name of the entrypoint
        package (str): The package path to access the entrypoint function
        func (str): The name of the function
    """
    equals = entrypoint.count('=')
    colons = entrypoint.count('=')

    assert equals == 1 and colons == 1, RuntimeError(
        'Invalid entrypoint format: "{}" Expected: '
        '"alias = package.module:function"'
        .format(entrypoint)
    )

    name, path = map(str.strip, entrypoint.split('='))
    package, func = path.split(':')
    return name, package, func


def mapped(entrypoints):
    """
    Get the list of entrypoints from the setup script and convert it
    into a map from name to package path and function tuple

    Arguments:
         entrypoints (list[str]): The list of entrypoints defined in the
            setup.py script.

    Returns:
        map (dict[str, tuple[str, str, str]): Mapping from name of the entrypoint
            to the name, package path, and function tuple.
    """
    result = dict()
    for entrypoint in entrypoints:
        name, package, func = parse(entrypoint)
        result[name] = (name, package, func)
    return result



from __future__ import print_function


def training_function():
    """
    Define a training function that just prints out to stdout. Training jobs
    do not take any arguments as input and do not need to return anything.

    Note that the function can be named anything as long as it is correctly
    referenced in the setup.py
    """
    print('Hello World')


def hosted_function(payload):
    """
    Define a hosted function that echos out the input. When creating a
    predict image, this is the function that is hosted on the invocations
    endpoint.

    Arguments:
        payload (dict[str, object]): This is the payload that will eventually
            be sent to the SageMaker server using a POST request to the
            `invocations` route.

    Returns:
        payload (dict[str, object]): The output of the function is expected to
            be either a dictionary (like the function input) or a JSON string.
    """
    return payload




from setuptools import setup, find_packages

setup(
    name='tensorflow_mnist',
    version='0.0.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'tensorflow',
    ],
    python_version='3.6',
    entry_points={
        'baklava.train': [
            'train = tensorflow_mnist.train:main',
        ],
        'baklava.predict': [
            'predict = tensorflow_mnist.predict:main',
        ]
    }
)




from typing import Dict, Optional, Sequence, Tuple

from google.cloud import aiplatform

def create_endpoint(
    project: str,
    instance: str,
    model: str,
    job_name: str,
    service_account: str,
    region: str
):
    aiplatform.init(project=project, location=region)

    # create model
    # container_image="gcr.io/spry-firefly-317822/predict-image",

    endpoint = aiplatform.Endpoint.create(
        display_name=job_name, project=project, location=region,
    )
    model_uri = f'{model}'
    print(f'model_uri: {model_uri}')
    model_object = aiplatform.Model(model_uri)

    endpoint.deploy(
        model=model_object,
        deployed_model_display_name = f'{job_name}-{model}',
        machine_type = instance,
        service_account = service_account

    )

print(create_endpoint(
    region='us-central1',
    instance="n1-standard-4",
    project="spry-firefly-317822", 
    job_name="sk_learn_example",
    model='5101602011485306880',
    service_account='alex-mbp@spry-firefly-317822.iam.gserviceaccount.com'
))


from setuptools import setup, find_packages

setup(
    name='simple_functions',
    version='0.0.1',
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'baklava.train': [
            'my_train = simple_functions.main:training_function',
        ],
        'baklava.predict': [
            'my_predict = simple_functions.main:hosted_function',
        ]
    }
)



$entrypoint
import argparse
import os
from mlsriracha.train import TrainAdapter

print('Starting mlctl container')

parser = argparse.ArgumentParser(
                description='Accept general args.'
                )
                
# https://gist.github.com/fralau/061a4f6c13251367ef1d9a9a99fb3e8d
parser.add_argument("--set",
    metavar="KEY=VALUE",
    nargs='*',
    help="Set a number of key-value pairs "
        "(do not put spaces before or after the = sign). "
        "If a value contains spaces, you should define "
        "it with double quotes: "
        'foo="this is a sentence". Note that '
        "values are always treated as strings.")

args, remainder = parser.parse_known_args()

def parse_var(s):
    """
    Parse a key, value pair, separated by '='
    That's the reverse of ShellArgs.

    On the command line (argparse) a declaration will typically look like:
        foo=hello
    or
        foo="hello world"
    """
    items = s.split('=')
    key = items[0].strip() # we remove blanks around keys, as is logical
    if len(items) > 1:
        # rejoin the rest:
        value = '='.join(items[1:])
    return (key, value)


def parse_vars(items):
    """
    Parse a series of key-value pairs and return a dictionary
    """
    d = {}

    if items:
        for item in items:
            key, value = parse_var(item)
            d[key] = value
    return d

if args.set is not None:
    # parse the key-value pairs
    param_values = parse_vars(args.set)

    print(f'params= {param_values}')
    data_fields = [
        # mlctl specific
        'sriracha_provider', 
        # Azure ML specific
        'training-data', 'validation-data', 'test-data', 'sriracha_mlflow_tracking_uri']
    for data_field in data_fields:
        if data_field in param_values.keys():
            print(f'{data_field}={param_values[data_field]} found in params')
            os.environ[data_field] = param_values[data_field]

# create training adapter env
ta = TrainAdapter(os.getenv('sriracha_provider'))
try:
    entrypoint(ta)
    ta.finish()
except TypeError:
    # TODO: TypeError should be only on entrypoint. Right now any TypeError will trigger
    print('User provided function does not support TrainingAdapter')
    entrypoint()


"""
Paths
=====
Define SageMaker path locations for both training and prediction containers.

https://docs.aws.amazon.com/sagemaker/latest/dg/your-algorithms-training-algo.html
"""
import os


def model(filename):
    """
    The path to model artifacts.

    Your algorithm should write all final model artifacts to this directory.
    Amazon SageMaker copies this data as a single object in compressed tar
    format to the S3 location that you specified in the CreateTrainingJob
    request. If multiple containers in a single training job write to this
    directory they should ensure no file/directory names clash. Amazon SageMaker
    aggregates the result in a tar file and uploads to S3.

    Arguments:
        filename (str): The name of the file which will be written back to S3

    Returns:
        path (str): The absolute path to the model output directory
    """
    return os.path.join(os.sep, 'opt', 'ml', 'model', filename)


def input(channel, filename):
    """
    The path to input artifacts.

    Amazon SageMaker allows you to specify "channels" for your docker container.
    The purpose of a channel is to copy data from S3 to a specified directory.

    Amazon SageMaker makes the data for the channel available in the
    /opt/ml/input/data/channel_name directory in the Docker container.

    For example, if you have three channels named training, validation, and
    testing, Amazon SageMaker makes three directories in the Docker container:

        /opt/ml/input/data/training
        /opt/ml/input/data/validation
        /opt/ml/input/data/testing

    Arguments:
        channel (str): The name of the channel which contains the given filename
        filename (str): The name of the file within a specific channel

    Returns:
        path (str): The absolute path to the specified channel file
    """
    return os.path.join(os.sep, 'opt', 'ml', 'input', 'data', channel, filename)


def failure():
    """
    The path to the failure file.

    If training fails, after all algorithm output (for example, logging)
    completes, your algorithm should write the failure description to this file.

    Returns:
        path (str): The absolute path to the failure file
    """
    return os.path.join(os.sep, 'opt', 'ml', 'output', 'failure')


def output(filename):
    """
    The path to the output artifacts.

    Your algorithm should write all final model artifacts to this directory.
    Amazon SageMaker copies this data as a single object in compressed tar
    format to the S3 location that you specified in the CreateTrainingJob
    request. If multiple containers in a single training job write to this
    directory they should ensure no file/directory names clash. Amazon SageMaker
    aggregates the result in a tar file and uploads to S3.

    Arguments:
        filename (str): The name of the file which will be written back to S3

    Returns:
        path (str): The absolute path to the model output directory
    """
    return os.path.join(os.sep, 'opt', 'ml', 'model', filename)


def config():
    """
    The path to all SageMaker configuration files.

    The directory where standard SageMaker configuration files are located
    SageMaker training creates the following files in this folder when training
    starts:

        - `hyperparameters.json`: Amazon SageMaker makes the hyperparameters in
            a CreateTrainingJob request available in this file.
        - `inputdataconfig.json`: You specify data channel information in the
            InputDataConfig parameter in a CreateTrainingJob request. Amazon
            SageMaker makes this information available in this file.
        - `resourceconfig.json`: The name of the current host and all host
            containers in the training

    Returns:
        path (str): The absolute path to the config directory
    """
    return os.path.join(os.sep, 'opt', 'ml', 'input', 'config')


def hyperparameters():
    """
    The path to the model hyperparameter configuration file.

    Amazon SageMaker makes the hyperparameters in a CreateTrainingJob request
    available in this file.

    Returns:
        path (str): The absolute path to the hyperparameters JSON
    """
    return os.path.join(config(), 'hyperparameters.json')


def input_data_config():
    """
    The path to the input data configuration file.

    You specify data channel information in the InputDataConfig parameter in a
    CreateTrainingJob request. Amazon SageMaker makes this information available
    in this file.

    Returns:
        path (str): The absolute path to the input data config JSON
    """
    return os.path.join(config(), 'inputdataconfig.json')


def resource_config():
    """
    The path to the resource configuration file.

    A file containing the name of the current host and all host containers in
    the training.

    Returns:
        path (str): The absolute path to the resource config JSON
    """
    return os.path.join(config(), 'resourceconfig.json')



from setuptools import setup, find_packages

setup(
    name='predict_initializer',
    version='0.0.1',
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'baklava.initialize': [
            'init = predict_initializer.main:load_model',
        ],
        'baklava.predict': [
            'pred = predict_initializer.main:predict',
        ]
    }
)



"""
Predict
=======
Functions used to load models at prediction time and produce model decisions.
"""
from __future__ import print_function

import functools

from sagemaker_paths import paths


@functools.lru_cache()
def load_model():
    """
    Loads and caches the model data.

    Note: Using `functools.lru_cache` is good practice for long operations that
    you want to run only once. This is especially useful when loading model
    artifacts.

    Returns:
        content (str): The model string
    """
    path = paths.model('model.txt')
    with open(path, 'r') as stream:
        content = stream.read()
    return content


def main(payload):
    """
    Defines a hosted function that appends the model.txt text to the data
    parameter.

    Note: This is the prediction entrypoint used by baklava!

    Arguments:
        payload (dict[str, object]): This is the payload that will eventually
            be sent to the SageMaker server using a POST request to the
            `invocations` route.

    Returns:
        payload (dict[str, object]): The output of the function is expected to
            be either a dictionary (like the function input) or a JSON string.
    """

    data = payload['data']
    result = str(data) + load_model()
    payload['data'] = result
    return payload




import boto3
client = boto3.Session().client('sagemaker')

response = client.create_model(
    ModelName='stumblebum-lenitives',
    PrimaryContainer={
        'Image': '436885317446.dkr.ecr.us-east-1.amazonaws.com/mlctl-test:predict-image',
        'ImageConfig': {
            'RepositoryAccessMode': 'Platform',
        },
        'Mode': 'SingleModel',
        'ModelDataUrl': 's3://mlctltest/example1_output/mlctl-stumblebum-lenitives/output/model.tar.gz',
        'Environment': {
            'sriracha_provider': 'awssagemaker'
        },
    },
    ExecutionRoleArn='arn:aws:iam::436885317446:role/sm-execution',
    Tags=[
        {
            'Key': 'mlctl',
            'Value': 'asdf'
        },
    ]
)

print(response)

response = client.create_endpoint_config(
    EndpointConfigName='stumblebum-lenitives-endpoint-config',
    ProductionVariants=[
        {
            'VariantName': 'main',
            'ModelName': 'stumblebum-lenitives',
            'InitialInstanceCount': 1,
            'InstanceType': 'ml.t2.medium',
            'InitialVariantWeight': 1.0,
        },
    ],
    # DataCaptureConfig={
    #     'EnableCapture': True|False,
    #     'InitialSamplingPercentage': 123,
    #     'DestinationS3Uri': 'string',
    #     'KmsKeyId': 'string',
    #     'CaptureOptions': [
    #         {
    #             'CaptureMode': 'Input'|'Output'
    #         },
    #     ],
    #     'CaptureContentTypeHeader': {
    #         'CsvContentTypes': [
    #             'string',
    #         ],
    #         'JsonContentTypes': [
    #             'string',
    #         ]
    #     }
    # },
    Tags=[
        {
            'Key': 'mlctl',
            'Value': 'asdf'
        },
    ],
)

print(response)


"""
Predict
=======
Functions used to load models at prediction time and produce model decisions.
"""
from __future__ import print_function

import pickle
import pkgutil
import json

import numpy as np
from non_python_files import paths


class cache(object):
    """
    Simplified replacement of functools.lur_cache for python 2.7

    Use functools.lru_cache in any version of python > 3.5
    """
    def __init__(self, func):
        self.func = func
        self.result = None

    def __call__(self, *args, **kwargs):
        if self.result:
            return self.result
        self.result = self.func()
        return self.result


@cache
def load_model():
    """
    Load the pickled model back into memory. Since loading models can be
    expensive, this function is cached so that the model only needs to be loaded
    into memory once.

    Returns:
        model (sklearn.tree.DecisionTreeClassifier): The trained decision tree.
    """
    with open(paths.model('model.pkl'), 'rb') as stream:
        return pickle.load(stream)


@cache
def load_thresholds():
    """
    Load the minimum and maximum age thresholds

    Returns:
        tuple:
            min_age (int): The minimum age to consider
            max_age (int): The maximum age to consider
    """
    content = pkgutil.get_data('non_python_files.resources', 'thresholds.json')
    mapping = json.loads(content)
    return mapping['min_age'], mapping['max_age']


def main(payload):
    """
    Run the decision tree classifier on the input payload. It is expected that
    the payload will have an 'age' and 'height' key, otherwise an error will
    occur.

    Note: This is the prediction entrypoint used by baklava!

    Arguments:
        payload (dict[str, object]): This is the payload that will eventaully
            be sent to the server using a POST request.

    Returns:
        payload (dict[str, object]): The output of the function is expected to
            be either a dictionary (like the function input) or a JSON string.
    """

    # Extract parameters from input dictionary
    age = payload['age']
    height = payload['height']

    # Return unknown weight if outside of normal ranges
    min_age, max_age = load_thresholds()
    if age > max_age:
        return {'weight': 'Unknown'}

    if age < min_age:
        return {'weight': 'Unknown'}

    # Load the model and execute
    model = load_model()
    x = np.array([[age, height]])  # Model expects matrix
    weights = model.predict(x)
    weight = int(weights[0])  # Model produces vector

    # Return back a payload with the result
    result = {'weight': weight}
    return result



"""
Render
======
Utilities for rendering folder templates using the core library string.Template
class.

Note: This module functions similar to the cookiecutter package except that it
does not have any external dependencies.
"""
import os
import string


def validate_directory(directory):
    """
    Raise an error if the directory does not exist or is not a directory.

    Args:
        directory (str): The path to the directory to check.
    """
    if not os.path.exists(directory):
        raise ValueError("Directory '{}' does not exist".format(directory))
    if not os.path.isdir(directory):
        raise ValueError("File '{}' is not a directory".format(directory))


def files(root):
    """
    Recursively get filename and directory paths from a root directory.

    Args:
        root (str): The root directory to get files from.

    Returns:
        filenames (list[str]): The collection of filenames and directories
            under the root directory.
    """
    filenames = list()
    for root, dirs, files in os.walk(root):
        for directory in dirs:
            if "__pycache__" in directory:
                continue
            filenames.append(os.path.join(root, directory))
        for name in files:
            if name.endswith(".pyc"):
                continue
            filenames.append(os.path.join(root, name))
    return filenames


def copy(source, destination, **replacements):
    """
    Recursively copy the contents of the source directory to the destination
    directory and replaces all template strings in both the file names and
    file contents.

    See: https://docs.python.org/3/library/string.html#template-strings

    Args:
        source (str): The path to the template root directory to read from.
        destination (str): The path to the directory to write to.

    Kwargs:
        replacements: A mapping from $-prefixed token to substitution.

    Returns:
        files (list[str]): The filenames in the destination folder.
    """
    # Validate that the source/destination directories exist
    validate_directory(source)
    validate_directory(destination)

    # Recursively get source files from template root
    sources = files(source)

    # Build destination paths from source paths
    destinations = list()
    for src in sources:

        # Strip off root
        size = len(source)
        relative = src[size:].lstrip(os.path.sep)

        # Substitute relative path filename
        result = string.Template(relative).safe_substitute(**replacements)
        destinations.append(os.path.join(destination, result))

    # Now loop through source files and write rendered files
    for src, dst in zip(sources, destinations):

        # Create the directory if source was a directory
        if os.path.isdir(src):
            os.makedirs(dst)
            continue

        # Write rendered files
        with open(src, "r") as template:
            with open(dst, "w") as rendered:
                content = template.read()
                result = string.Template(content).safe_substitute(**replacements)
                rendered.write(result)

    return destinations



import os
import baklava


def sample():
    directory = os.path.dirname(__file__)
    return os.path.join(directory, "sample")


def test_predict():
    result = baklava.predict(sample(), ["--help"])
    assert result == 0


def test_train():
    result = baklava.predict(sample(), ["--help"])
    assert result == 0


if __name__ == '__main__':
    test_predict()
    test_train()



from setuptools import setup, find_packages

setup(
    name='multiple_images',
    version='0.0.1',
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'baklava.train': [
            'first = multiple_images.main:train1',
            'second = multiple_images.main:train1',
            'third = multiple_images.main:train2',
        ],
        'baklava.predict': [
            'third = multiple_images.main:predict1',
            'fourth = multiple_images.main:predict1',
            'fifth = multiple_images.main:predict2',
        ]
    }
)



import os
import uuid
import contextlib
import tempfile

from baklava import pypi


@contextlib.contextmanager
def pushenv():
    original = os.environ.copy()
    yield
    os.environ = original


def test_load_env():

    with pushenv():
        # Set environment variables (Top Precedence)
        index = str(uuid.uuid4())
        extra = str(uuid.uuid4())
        os.environ['PIP_INDEX_URL'] = index
        os.environ['PIP_EXTRA_INDEX_URL'] = extra

        # API Call
        config = pypi.load()

        # Assertions
        assert config['index-url'] == index
        assert config['extra-index-url'] == extra


def test_invalid_config_path():
    with pushenv():
        filename = str(uuid.uuid4())
        os.environ["PIP_CONFIG_FILE"] = filename
        config = pypi.load()
        assert config['config-file'] == filename


def test_invalid_config_content():

    temp = tempfile.NamedTemporaryFile()

    with open(temp.name, 'wb') as handle:
        handle.write(b"\001")

    with pushenv():
        os.environ["PIP_CONFIG_FILE"] = temp.name
        config = pypi.load()
        assert config['config-file'] == temp.name


if __name__ == '__main__':
    test_load_env()
    test_invalid_config_path()
    test_invalid_config_content()



"""
Commands
========
Setuptools entrypoints for building training/prediction docker containers from
distributions.
"""

from distutils.core import Command
import os

from mlbaklava import images, entrypoint, distribution


class Docker(Command):

    def build(self, artifacts, entry, directory):
        # Exit if we do not want to build
        if self.nobuild:
            return

        # Check for docker before building
        if not images.check_docker():
            return

        # Extract distribution parameters for building
        name = self.distribution.metadata.name
        version = self.distribution.metadata.version

        # Come up with name and tag for the resulting image
        if self.entrypoint is not None:
            entry_name, entry_package, entry_func = entry
            name = '{}-{}'.format(name, entry_name)
        tag = images.tag(artifacts, name, version)

        # Build and tag the image
        image = images.build(directory, name=tag)

        # Save the identity file if a name is provided
        if self.idfile:
            with open(self.idfile, 'w') as stream:
                stream.write(image.id)

        if self.tag is not None:
            images.rename(image.id, self.tag)

        if self.hash:
            print(image.id)


class Train(Docker):
    description = 'create a docker image for local training'
    user_options = [

        # Distribution Configs
        ('entrypoint=', 'e', "The project entrypoint"),
        ('provider=', 'p', "The provider this container should be built for"),

        # Docker Configs
        ('idfile=', 'f', "Save image name to the given file"),
        ('tag=', 't', "Tag the image with a string"),

        # Flags
        ('nobuild', 'b', "Flag to not build the image"),
        ('hash', 'h', "Silently build and output image hash"),
    ]

    def initialize_options(self):

        # Distribution Configs
        self.entrypoint = None
        self.pyver = None
        self.provider = None

        # Docker Configs
        self.idfile = None
        self.tag = None

        # Flags
        self.nobuild = False
        self.hash = False

    def finalize_options(self):
        pass

    def run(self):

        # Get the entrypoint
        entry = entrypoint.get(
            entry=self.distribution.entry_points,
            key='mlbaklava.train',
            name=self.entrypoint
        )

        # Create the source distribution
        self.run_command('sdist')
        sdist = self.get_finalized_command('sdist')
        assert len(sdist.archive_files), RuntimeError(
            'Expected a single archive to be created from source distribution'
        )

        # Extract path to the distribution
        path = sdist.archive_files[0]
        directory = sdist.dist_dir
        archive = os.path.relpath(path, directory)

        # Extract distribution configurations
        requirements = self.distribution.install_requires
        python_version = self.distribution.python_version
        dockerlines = self.distribution.dockerlines

        # Build docker distribution files
        artifacts = distribution.train(
            path=directory,
            archive=archive,
            entrypoint=entry,
            requirements=requirements,
            python_version=python_version,
            dockerlines=dockerlines,
            provider=self.provider
        )

        self.build(artifacts, entry, directory)

class Process(Docker):
    description = 'create a docker image for local processing'
    user_options = [

        # Distribution Configs
        ('entrypoint=', 'e', "The project entrypoint"),
        ('provider=', 'p', "The provider this container should be built for"),

        # Docker Configs
        ('idfile=', 'f', "Save image name to the given file"),
        ('tag=', 't', "Tag the image with a string"),

        # Flags
        ('nobuild', 'b', "Flag to not build the image"),
        ('hash', 'h', "Silently build and output image hash"),
    ]

    def initialize_options(self):

        # Distribution Configs
        self.entrypoint = None
        self.pyver = None
        self.provider = None

        # Docker Configs
        self.idfile = None
        self.tag = None

        # Flags
        self.nobuild = False
        self.hash = False

    def finalize_options(self):
        pass

    def run(self):

        # Get the entrypoint
        entry = entrypoint.get(
            entry=self.distribution.entry_points,
            key='mlbaklava.process',
            name=self.entrypoint
        )

        # Create the source distribution
        self.run_command('sdist')
        sdist = self.get_finalized_command('sdist')
        assert len(sdist.archive_files), RuntimeError(
            'Expected a single archive to be created from source distribution'
        )

        # Extract path to the distribution
        path = sdist.archive_files[0]
        directory = sdist.dist_dir
        archive = os.path.relpath(path, directory)

        # Extract distribution configurations
        requirements = self.distribution.install_requires
        python_version = self.distribution.python_version
        dockerlines = self.distribution.dockerlines

        # Build docker distribution files
<<<<<<< HEAD:mlbaklava/commands.py
        artifacts = distribution.process(
=======
        artifacts = distribution.processing(
>>>>>>> c2a48febc82dbf6a0023b94eb84c20919db4a3c6:baklava/commands.py
            path=directory,
            archive=archive,
            entrypoint=entry,
            requirements=requirements,
            python_version=python_version,
            dockerlines=dockerlines,
            provider=self.provider
        )

        self.build(artifacts, entry, directory)


class Deploy(Docker):
    description = 'create a docker image for local prediction'
    user_options = [

        # Distribution Configs
        ('entrypoint=', 'e', "The project entrypoint"),
        ('initializer=', 'n', "The project initializer"),
        ('workers=', 'e', "The number of worker processes host in the image (default=8)"),
        ('provider=', 'p', "The provider to create this container for"),

        # Image Configs
        ('idfile=', 'f', "Save image name to the given file"),
        ('tag=', 't', "Tag the image with a string"),

        # Flags
        ('nobuild', 'b', "Flag to not build the image"),
        ('hash', 'h', "Silently build and output image hash"),
    ]

    def initialize_options(self):

        # Distribution Configs
        self.entrypoint = None
        self.initializer = None
        self.workers = None
        self.provider = None

        # Docker Configs
        self.idfile = None
        self.tag = None

        # Flags
        self.nobuild = False
        self.hash = False

    def finalize_options(self):
        pass

    def run(self):

        # Get the initializer
        initializer = (None, None, None)
        if self.initializer or 'mlbaklava.initialize' in self.distribution.entry_points:
            initializer = entrypoint.get(
                entry=self.distribution.entry_points,
                key='mlbaklava.initialize',
                name=self.initializer
            )

        # Get the entrypoint
        entry = entrypoint.get(
            entry=self.distribution.entry_points,
            key='mlbaklava.deploy',
            name=self.entrypoint
        )

        # Create the source distribution
        self.run_command('sdist')
        sdist = self.get_finalized_command('sdist')
        assert len(sdist.archive_files), RuntimeError(
            'Expected a single archive to be created from source distribution'
        )

        # Extract path to the distribution
        path = sdist.archive_files[0]
        directory = sdist.dist_dir
        archive = os.path.relpath(path, directory)

        # Extract distribution configurations
        requirements = self.distribution.install_requires
        python_version = self.distribution.python_version
        dockerlines = self.distribution.dockerlines

        # Build docker distribution files
        artifacts = distribution.deploy(
            path=directory,
            archive=archive,
            entrypoint=entry,
            requirements=requirements,
            initializer=initializer,
            python_version=python_version,
            dockerlines=dockerlines,
            provider=self.provider,
            workers=self.workers
        )

        self.build(artifacts, entry, directory)


def passthrough(dist, keyword, value):
    """
    This function is used to register distutils keywords
    """
    pass



"""
Train
=====
Defines functions which train models and write model artifacts to disk.
"""
from __future__ import print_function

import pickle

import pandas as pd
from sklearn.tree import DecisionTreeRegressor

from non_python_files import paths


def train(x, y):
    """
    Train a decision tree regressor using a floating point feature matrix and
    a regression target.

    Arguments:
        x (~np.ndarray): The user feature matrix. (n_samples, n_features)
        y (~np.ndarray): The regression target. (n_samples, 1)

    Returns:
        model (sklearn.tree.DecisionTreeRegressor): The trained decision tree.
    """
    model = DecisionTreeRegressor(min_samples_leaf=1)
    model.fit(x, y)
    return model


def main():
    """
    Load features and labels, train the tree, and serialize model artifact.

    Note: This is the training entrypoint used by baklava!
    """
    # Read in the training data
    path = paths.input('training', 'data.csv')
    data = pd.read_csv(path)

    # Extract features and labels
    x = data[['age', 'height']]
    y = data['weight']

    # Fit the model to the data
    model = train(x, y)

    # Save model object using pickle format
    with open(paths.output('model.pkl'), 'wb') as stream:
        pickle.dump(model, stream)

    print('Success!')



"""
Model
=====
Defines the model architecture used during training and prediction
"""
import tensorflow as tf


def build():
    """
    Builds a model graph which can be trained to compute MNIST predictions.

    Returns:
        graph (~tf.Graph): The tensorflow graph which contains all operations.
        x (~tf.Tensor): A placeholder which acts as the feature input
        y (~tf.Tensor): A placeholder which acts as the label input
        initializer (~tf.Tensor): A tensor which initializes all variables
        accuracy (~tf.Tensor): A tensor which computes the accuracy for the
            current input data. Requires that values are fed into both x and y.
        prediction (~tf.Tensor): A tensor which computes the predicted value
            for y.
    """

    graph = tf.Graph()
    with graph.as_default():

        # Define global inputs
        y = tf.placeholder(tf.float32, [None, 10])
        x = tf.placeholder(tf.float32, [None, 784])

        # Peform matrix multiplication
        logits = tf.layers.dense(x, 10)

        # Define your loss function
        cross_entropy = tf.nn.softmax_cross_entropy_with_logits_v2(
            labels=y,
            logits=logits
        )
        loss = tf.reduce_mean(cross_entropy)

        # Metrics
        correct_prediction = tf.equal(tf.argmax(logits, 1), tf.argmax(y, 1))
        accuracy = tf.reduce_mean(tf.cast(correct_prediction, tf.float32))

        prediction = tf.squeeze(tf.argmax(logits, 1))

        # A single optimization step
        step = tf.train.GradientDescentOptimizer(0.5).minimize(loss)

        # Graph initialization
        initializer = tf.global_variables_initializer()

    return graph, x, y, step, initializer, accuracy, prediction



import re

from baklava import environment


def test_get_environment_variables():
    mapping = environment.get_environment_variables()

    # Check that the keys are properly generated. We can't know for sure in each
    # environment what the values will be
    assert 'pip_index_url' in mapping
    assert 'pip_trusted_host' in mapping
    assert 'pip_extra_index_url' in mapping


def test_get_python_version():
    version = environment.get_python_version()
    assert re.match(r'\d\.\d', version) is not None



"""
Train
=====
Defines functions which train models and write model artifacts to disk.
"""
from __future__ import print_function

from sagemaker_paths import paths


def process(data):
    """
    Lower cases and reverses a string

    Arguments:
        data (str): The string to transform

    Returns:
        result (str): The lower cased, reversed string
    """
    return data.lower()[::-1]


def main():
    """
    Use the paths to an expected set of training data and the user-defined
    reverse function to reverse the training data contents.

    Note: This is the training entrypoint used by baklava!
    """

    # Read in the training data
    path = paths.input('training', 'train.txt')
    with open(path, 'r') as stream:
        content = stream.read()

    # Process the string training data
    result = process(content)

    # Write results back
    path = paths.model('model.txt')
    with open(path, 'w') as stream:
        stream.write(result)

    print('Success!')


if __name__ == '__main__':
    print(process('dlRow OlLeh'))



import os
import shutil
import tempfile
import contextlib

from baklava import render


@contextlib.contextmanager
def tempdir():
    path = tempfile.mkdtemp()
    yield path
    shutil.rmtree(path)


def test_example_template():

    name = 'test'
    value = '1'

    with tempdir() as path:

        source = os.path.join(os.path.dirname(__file__), 'template')

        render.copy(source, path, name=name, value=value)

        directory = os.path.join(path, name)
        assert os.path.isdir(directory)

        filename = os.path.join(directory, "{}.txt".format(name))
        assert os.path.exists(filename)

        with open(filename) as stream:
            data = stream.read()

        assert value == data


if __name__ == '__main__':
    test_example_template()



from setuptools import setup, find_packages

setup(
    name='sagemaker_paths',
    version='0.0.1',
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'baklava.train': [
            'train = sagemaker_paths.train:main',
        ],
        'baklava.predict': [
            'predict = sagemaker_paths.predict:main',
        ]
    }
)



from google.cloud import aiplatform
from google.protobuf import json_format
from google.protobuf.struct_pb2 import Value
from google.cloud.aiplatform_v1.types import InputDataConfig, GcsDestination
def create_training_job(
    container_image: str,
    instance: str,
    region: str,
    project: str,
    job_name: str,
    output_data: str,
    input_data: str
):
    client = aiplatform.gapic.PipelineServiceClient(client_options= {"api_endpoint": f"{region}-aiplatform.googleapis.com"})
    training_task_inputs_dict = {
        "workerPoolSpecs": [
            {
                "replicaCount": 1,
                "machineSpec": {"machineType": instance},
                "containerSpec": {
                    # A working docker image can be found at gs://cloud-samples-data/ai-platform/mnist_tfrecord/custom_job
                    "imageUri": container_image,
                    "env": [{"name": "sriracha_provider", "value": "gcpvertex"}
                    ],
                },
            }
        ],
        "baseOutputDirectory": {
            # The GCS location for outputs must be accessible by the project's AI Platform service account.
            "output_uri_prefix": output_data
        },
    }
    training_task_inputs = json_format.ParseDict(training_task_inputs_dict, Value())

    # Accessible at https://console.cloud.google.com/storage/browser/google-cloud-aiplatform/schema/trainingjob/definition;tab=objects?prefix=&forceOnObjectsSortingFiltering=false
    training_task_definition = "gs://google-cloud-aiplatform/schema/trainingjob/definition/custom_task_1.0.0.yaml"

    input_data_config = {
            "dataset_id": '1889198471030767616',
            "gcs_destination": {"output_uri_prefix": input_data}
        }

# when file is passed in, GCP does it's own test/valid/train data split

    training_pipeline = {
        "display_name": job_name,
        "training_task_definition": training_task_definition,
        "training_task_inputs": training_task_inputs,
        "input_data_config": input_data_config,
        # "model_to_upload": {
        #     "display_name": f"{job_name}-model",
        #     "container_spec": {"image_uri": serving_image_uri},
        # },
    }

    parent = f"projects/{project}/locations/{region}"
    response = client.create_training_pipeline(
        parent=parent, training_pipeline=training_pipeline
    )
    return response

region = 'us-central1'

aiplatform.init(
    # your Google Cloud Project ID or number
    # environment default used is not set
    project='spry-firefly-317822',

    # the Vertex AI region you will use
    # defaults to us-central1
    location='',

    # Googlge Cloud Stoage bucket in same region as location
    # used to stage artifacts
    staging_bucket='gs://vertex-staging',

    # custom google.auth.credentials.Credentials
    # environment default creds used if not set
    # credentials=my_credentials,

    # customer managed encryption key resource name
    # will be applied to all Vertex AI resources if set
    # encryption_spec_key_name=my_encryption_key_name,

    # the name of the experiment to use to track
    # logged metrics and parameters
    experiment='experiment-1',

    # description of the experiment above
    experiment_description='my experiment description'
)

print(create_training_job(
    region='us-central1',
    instance="n1-standard-4",
    project="spry-firefly-317822", 
    job_name="sk_learn_example",
    container_image="gcr.io/spry-firefly-317822/train-image",
    input_data='gs://mlctl-data/',
    output_data="gs://mlctl-model/vertex/"
))


from __future__ import print_function

import os
import tempfile
import re
import subprocess

import pytest
import docker
from docker.utils.json_stream import json_stream


import baklava


# ------------------------------------------------------------------------------
# Utilities
# ------------------------------------------------------------------------------

def example(name='1-simple-functions'):
    """
    Get the path to the example directory

    Args:
        name (str): The name of the example to get the path for

    Returns:
        path (str): The example directory path
    """
    return os.path.join(
        os.path.dirname(os.path.dirname(__file__)),
        'examples',
        name,
    )


def tmp():
    """
    Get the path to a new temporary file

    Returns:
        path (str): The temporary file path
    """

    return tempfile.NamedTemporaryFile(delete=False).name


# ------------------------------------------------------------------------------
# Basic Build
# ------------------------------------------------------------------------------

examples = [
    '1-simple-functions',
    '2-sagemaker-paths',
    '3-sklearn-tree',
    '4-tensorflow-mnist',
    '6-internal-dependencies',
    '7-custom-docker-lines',
    '9-non-python-files',
]


@pytest.mark.slow
@pytest.mark.parametrize("name", examples)
def test_train_build(name):
    """Check that basic train image builds successfully"""
    code = baklava.train(example(name), [])
    assert code == 0


@pytest.mark.slow
@pytest.mark.parametrize("name", examples)
def test_predict_build(name):
    """Check that basic predict image builds successfully"""
    code = baklava.predict(example(name), [])
    assert code == 0


# ------------------------------------------------------------------------------
# 1-simple-functions
# ------------------------------------------------------------------------------

@pytest.mark.slow
def test_1_simple_functions_train_idfile():
    """Check that the id file can be extracted from build procedure"""

    idfile = tmp()
    code = baklava.train(example(), ['--idfile={}'.format(idfile)])

    assert code == 0

    with open(idfile) as stream:
        content = stream.read()

    # The content of the idfile will look like the docker sha checksum:
    # sha256:e422a43d58ceccc77283145bdb42312461cb67fd90e66bc5bc2fe59af51bdf8c
    assert re.match("sha256\:[0-9a-f]{64}", content) is not None


@pytest.mark.slow
def test_1_simple_functions_train_execute():
    """Check that the docker image executes successfully"""

    process = baklava.api.run(
        path=example(),
        cmd='train',
        args=['-t', 'test-image']
    )
    assert process.returncode == 0, 'Failed to build image'

    client = docker.from_env()
    response = client.containers.run('test-image')

    stdout = response.decode('utf8')
    assert re.search("Hello World", stdout) is not None


# ------------------------------------------------------------------------------
# 5-multiple-images
# ------------------------------------------------------------------------------

@pytest.mark.slow
@pytest.mark.parametrize("entrypoint, expected", [
    ('first', 'Train 1!'),
    ('second', 'Train 1!'),
    ('third', 'Train 2!'),
])
def test_5_multiple_images_train_execute(entrypoint, expected):
    process = baklava.api.run(
        path=example('5-multiple-images'),
        cmd='train',
        args=['--entrypoint', entrypoint, '-t', 'test-image'],
    )
    assert process.returncode == 0, 'Failed to build image'

    client = docker.from_env()
    response = client.containers.run('test-image')

    stdout = response.decode('utf8')
    assert re.search(expected, stdout) is not None


# ------------------------------------------------------------------------------
# 7-custom-docker-lines
# ------------------------------------------------------------------------------

@pytest.mark.slow
def test_7_build_and_check():
    # Get the path to the artifacts
    dist = os.path.join(example('7-custom-docker-lines'), 'dist')

    # Build the image using low level API to get output directly
    client = docker.from_env()
    response = client.api.build(
        path=dist,
        quiet=False,
        nocache=True,
    )

    # Check to see that custom run command and its output exists
    found_command = False
    found_output = False
    for chunk in json_stream(response):
        if 'stream' in chunk:
            formatted = chunk['stream'].rstrip()

            # Check if we ever see the run command
            if re.search('RUN echo hello world$', formatted):
                found_command = True

            # If we saw the run command, check for output
            if found_command and re.search('^hello world$', formatted):
                found_output = True

    assert found_command
    assert found_output


@pytest.mark.slow
def test_7_custom_docker_lines_train():
    """Check that setup.py dockerlines works as expected for training"""

    # Build the distribution without building the image
    code = baklava.train(example('7-custom-docker-lines'), ['--nobuild'])
    assert code == 0

    test_7_build_and_check()


@pytest.mark.slow
def test_7_custom_docker_lines_predict():
    """Check that setup.py dockerlines works as expected for prediction"""

    # Build the distribution without building the image
    code = baklava.predict(example('7-custom-docker-lines'), ['--nobuild'])
    assert code == 0

    test_7_build_and_check()


# ------------------------------------------------------------------------------
# 8-predict-initializer
# ------------------------------------------------------------------------------

@pytest.mark.slow
def test_8_predict_initializer():
    """Test that the predict initializer example builds correctly"""

    code = baklava.predict(example('8-predict-initializer'), [])
    assert code == 0


# ------------------------------------------------------------------------------
# 9-non-python-files
# ------------------------------------------------------------------------------

@pytest.mark.slow
def test_9_non_python_files():
    """Check that setup.py python_version works as expected"""

    name = '9-non-python-files'
    code = baklava.train(example(name), ['--nobuild'])
    assert code == 0

    dist = os.path.join(example(name), 'dist')

    # Build the image using low level API to get output directly
    client = docker.from_env()
    response = client.api.build(
        path=dist,
        quiet=False,
        nocache=True,
    )

    # Check to see that custom run command and its output exists
    for chunk in json_stream(response):
        if 'stream' in chunk:
            formatted = chunk['stream'].rstrip()
            print(formatted)
            if re.search('FROM python:2\.7\.15-slim$', formatted):
                return

    # If we made it here, we did not find the correct python version
    assert False


# ------------------------------------------------------------------------------
# Train flags (Sanity Tests)
# ------------------------------------------------------------------------------

@pytest.mark.slow
def test_flag_train_nobuild():
    """Test that training image nobuild flag succeeds"""

    name = '1-simple-functions'
    code = baklava.train(example(name), args=['--nobuild'])
    assert code == 0


@pytest.mark.slow
def test_flag_train_idfile():
    """Test that training image idfile argument succeeds"""

    name = '1-simple-functions'
    code = baklava.train(example(name), args=['--idfile=id.txt'])
    assert code == 0


# ------------------------------------------------------------------------------
# Predict flags (Sanity Tests)
# ------------------------------------------------------------------------------

@pytest.mark.slow
def test_flag_predict_nobuild():
    """Test that predict image nobuild flag succeeds"""

    name = '1-simple-functions'
    code = baklava.predict(example(name), args=['--nobuild'])
    assert code == 0


@pytest.mark.slow
def test_flag_predict_idfile():
    """Test that predict image idfile argument succeeds"""

    name = '1-simple-functions'
    code = baklava.predict(example(name), args=['--idfile=id.txt'])
    assert code == 0


@pytest.mark.slow
def test_flag_predict_workers():
    """Test that predict image workers argument succeeds"""

    name = '1-simple-functions'
    code = baklava.predict(example(name), args=['--workers=10'])
    assert code == 0


if __name__ == '__main__':
    pytest.main([__file__, '--slow'])



"""
API
===
Python API for dockerizing python packages.
"""
import subprocess
import os
import sys


def run(path, cmd, args, stdout=None, stderr=None):
    """
    Runs a python packaging command on a specified directory

    .. note::
        This uses a subprocess call rather than importing the setup file
        directly because there is no guarantee how the setup file could be
        structured.

    Arguments:
        path (str): The path where the package directory and :literal:`setup.py`
            are located
        cmd (str): The specific setuptools command to run
        args (list[str]): The arguments to pass to the predict command. By
            default the system arguments are used.
        stdout (int): The subprocess option to choose how to
            capture the stdout stream (Should only be used for testing)
        stderr (int): The subprocess option to choose how to
            capture the stderr stream (Should only be used for testing)

    Returns:
        process (subprocess.Popen): The process that was run
    """
    path = os.path.abspath(path)

    args = [
        'python',
        '{path}/setup.py'.format(path=path),
        cmd,
    ] + args

    process = subprocess.Popen(args, cwd=path, stdout=stdout, stderr=stderr)
    process.wait()
    return process

def process(path=os.getcwd(), args=None):
    """
    Build a training container for a python package.

    Arguments:
        path (str): The path where the package directory and :literal:`setup.py`
            are located.
        args (list[str]): The arguments to pass to the train command. By
            default the system arguments are used.

    Returns:
        code (int): The process return code
    """
    return run(path, 'process', args).returncode

def train(path=os.getcwd(), args=None):
    """
    Build a training container for a python package.

    Arguments:
        path (str): The path where the package directory and :literal:`setup.py`
            are located.
        args (list[str]): The arguments to pass to the train command. By
            default the system arguments are used.

    Returns:
        code (int): The process return code
    """
    return run(path, 'train', args).returncode


def deploy(path=os.getcwd(), args=None):
    """
    Build a prediction container for a python package.

    Arguments:
        path (str): The path where the package directory and :literal:`setup.py`
            are located.
        args (list[str]): The arguments to pass to the predict command. By
            default the system arguments are used.

    Returns:
        code (int): The process return code
    """
    return run(path, 'deploy', args).returncode



from __future__ import print_function

import time
import functools


@functools.lru_cache()
def load_model():
    """
    This is an example function which takes a long time to load. In some cases
    models will need many seconds or minutes to load into memory. Rather than
    waiting for a model to load on the first function call, this function can
    be called immediately after the python process has been started. See the
    `setup.py` file for the initialization call.

    In this example, the "model" is just a constant number, but this could be
    any python object.

    Returns:
        result (int): A constant value to use a the model
    """
    print('Running Slow Load Function!')
    time.sleep(5)  # Artificial 5 second penalty to loading the model
    return 1


def predict(payload):
    """
    Define a hosted function that echos out the input. When creating a
    predict image, this is the function that is hosted on the invocations
    endpoint.

    This function just adds 1 to any object in the payload if it is possible

    Arguments:
        payload (dict[str, object]): This is the payload that will eventually
            be sent to the SageMaker server using a POST request to the
            `invocations` route.

    Returns:
        payload (dict[str, object]): The output of the function is expected to
            be either a dictionary (like the function input) or a JSON string.
    """

    print('Loading model...', end='')
    model = load_model()
    print('Done.')

    for key, value in payload.items():
        try:
            payload[key] = int(value) + model
        except ValueError:
            pass  # Do nothing if value can't be casted to int

    return payload




"""
Predict
=======
Functions used to load models at prediction time and produce model decisions.
"""
from __future__ import print_function

import functools

import numpy as np
import tensorflow as tf

from tensorflow_mnist import model, paths


@functools.lru_cache()
def load_model():
    """
    Load the saved model variables back into memory. Since loading models can be
    expensive, this function is cached so that the model only needs to be loaded
    into memory once.
    """

    # Recreate the model graph
    graph, x, y, step, initializer, accuracy, prediction = model.build()

    # Create a session which will hold variable state in memory
    session = tf.Session(graph=graph)
    path = paths.model('mnist')

    with graph.as_default():
        saver = tf.train.Saver()

        # Restore the graph variable state
        saver.restore(session, path)

    return session, x, prediction


def main(payload):
    """
    Run the tensorflow model on the input payload. It is expected that
    the payload will have a `data` node which contains pixel intensities

    Note: This is the prediction entrypoint used by baklava!

    Arguments:
        payload (dict[str, object]): This is the payload that will eventaully
            be sent to the server using a POST request.

    Returns:
        payload (dict[str, object]): The output of the function is expected to
            be either a dictionary (like the function input) or a JSON string.
    """

    # Extract parameters from input dictionary
    if 'data' not in payload:
        inputs = [0] * 784
    else:
        inputs = payload['data']

    data = np.array(inputs).reshape(1, 784)

    # Load model session and interface
    session, x, prediction = load_model()

    # Compute prediction
    prediction = int(session.run(prediction, feed_dict={x: data}))

    # Return back a payload with the result
    result = {'prediction': prediction}
    return result



"""
Distribution
============
Functions for building distribution artifacts prior to building docker images
"""
import pkg_resources
import os
import textwrap

from mlbaklava import environment, render

def format_containerentrypoint(job_type, provider):
    """
    Ensures containter entyrpoint is properly assigned. If this is running on Azure ML
    the container entrypoint is removed.

    Arguments:
        dockerlines (Union[str, list[str]]): The dockerlines provided by the
            python distribution setup file.

    Returns:
        dockerlines (str): The block of dockerlines to insert into the template
    """

    if job_type == 'train' and provider == 'azureml':
        print('Swapping container entrypoint for AzureML')
        return ""

    else:
        return 'ENTRYPOINT ["python", "/opt/main.py"]'

def format_dockerlines(dockerlines):
    """
    Ensures docker lines are correctly formatted.

    Users may input either a string or a list of strings in the setup.py. This
    method ensures that the format is correct regardless of input type.

    Arguments:
        dockerlines (Union[str, list[str]]): The dockerlines provided by the
            python distribution setup file.

    Returns:
        dockerlines (str): The block of dockerlines to insert into the template
    """
    if dockerlines in (None, "", []):
        return ""

    assert isinstance(dockerlines, (list, str)), TypeError(
        'Docker lines must be a list or string. Got {}'
        .format(type(dockerlines))
    )

    if isinstance(dockerlines, str):
        return textwrap.dedent(dockerlines)

    return '\n'.join(dockerlines)


def format_requirements(requirements):
    if requirements in (None, "", []):
        return ""

    return "RUN pip install --no-cache-dir {}".format(' '.join(requirements))


def format_entrypoints(entrypoint, alias):
    name, package, func = entrypoint
    if name is None:
        return "{alias} = None".format(alias=alias)
    return 'from {package} import {func} as {alias}'.format(package=package, func=func, alias=alias)


def build_parameters(archive, entrypoint, requirements, python_version, dockerlines, provider, job_type):

    # Load PyPI environment
    env = environment.get_environment_variables()

    # Set python version to environment python version
    if python_version is None:
        python_version = environment.get_python_version()

    # Make sure to generate the rgiht containerentrypoint
    containerentrypoint = format_containerentrypoint(job_type=job_type, provider=provider)

    # Make sure dockerlines are properly formatted
    dockerlines = format_dockerlines(dockerlines)
    requirements.append("mlsriracha>=0.0.1")
    requirements = format_requirements(requirements)
    entrypoint = format_entrypoints(entrypoint, 'entrypoint')

    # Write dockerfile
    return dict(
        python_version=python_version,
        distribution=archive,
        requirements=requirements,
        dockerlines=dockerlines,
        entrypoint=entrypoint,
        containerentrypoint=containerentrypoint,
        **env
    )


def train(path, archive, entrypoint, requirements, python_version, dockerlines, provider):
    """
    Create docker-specific training image artifacts

    Args:
        path (str): The directory to the distribution path to save artifacts to
        archive (str): The path to the source distribution
        entrypoint (str): The package entrypoint
        requirements (list[str]): The list of package requirements
        python_version (str): The specific version of python to use
        dockerlines (Union[str, list[str]]): Dockerlines provided by setup file

    Returns:
        artifacts (list[str]): The collection of artifacts used to create a
            training image.
    """
    # Common parameters
    parameters = build_parameters(archive, entrypoint, requirements, python_version, dockerlines, provider, 'train')

    # Render the result
    src = pkg_resources.resource_filename('mlbaklava.resources', 'train')
    files = render.copy(src, path, **parameters)

    # Return all distribution files
    files.append(os.path.join(path, archive))
    return files

def process(path, archive, entrypoint, requirements, python_version, dockerlines, provider):
    """
    Create docker-specific training image artifacts

    Args:
        path (str): The directory to the distribution path to save artifacts to
        archive (str): The path to the source distribution
        entrypoint (str): The package entrypoint
        requirements (list[str]): The list of package requirements
        python_version (str): The specific version of python to use
        dockerlines (Union[str, list[str]]): Dockerlines provided by setup file

    Returns:
        artifacts (list[str]): The collection of artifacts used to create a
            training image.
    """
    # Common parameters
    parameters = build_parameters(archive, entrypoint, requirements, python_version, dockerlines, provider, 'process')

    # Render the result
    src = pkg_resources.resource_filename('mlbaklava.resources', 'process')
    files = render.copy(src, path, **parameters)

    # Return all distribution files
    files.append(os.path.join(path, archive))
    return files


def deploy(path, archive, entrypoint, requirements, initializer, python_version, dockerlines, provider, workers=8):
    """
    Create docker-specific prediction image artifacts

    Args:
        path (str): The directory to the distribution path to save artifacts to
        archive (str): The path to the source distribution
        entrypoint (str): The package entrypoint
        requirements (list[str]): The list of package requirements
        initializer (str): The initization entrypoint to the package
        python_version (str): The specific version of python to use
        dockerlines (Union[str, list[str]]): Dockerlines provided by setup file
        workers (int): The number of worker processes to spawn in the image

    Returns:
        artifacts (list[str]): The collection of artifacts used to create a
            prediction image.
    """

    # Common parameters
    parameters = build_parameters(archive, entrypoint, requirements, python_version, dockerlines, provider, 'predict')

    # Update parameters with options
    initializer = format_entrypoints(initializer, 'initializer')

    parameters.update(
        initializer=initializer,
        workers=workers,
    )

    # Render the result
    src = pkg_resources.resource_filename('mlbaklava.resources', 'deploy')
    files = render.copy(src, path, **parameters)

    # Return all distribution files
    files.append(os.path.join(path, archive))
    return files



"""
Predict
=======
Functions used to load models at prediction time and produce model decisions.
"""
from __future__ import print_function

import pickle
import functools

import numpy as np

@functools.lru_cache()
def init(pa):
    """
    Load the pickled model back into memory. Since loading models can be
    expensive, this function is cached so that the model only needs to be loaded
    into memory once.

    Returns:
        model (sklearn.tree.DecisionTreeClassifier): The trained decision tree.
    """
    with open(pa.model_artifact('model.pkl'), 'rb') as stream:
        return pickle.load(stream)


def main(pa, payload):
    """
    Run the decision tree classifier on the input payload. It is expected that
    the payload will have an 'age' and 'height' key, otherwise an error will
    occur.

    Note: This is the prediction entrypoint used by baklava!

    Arguments:
        payload (dict[str, object]): This is the payload that will eventaully
            be sent to the server using a POST request.

    Returns:
        payload (dict[str, object]): The output of the function is expected to
            be either a dictionary (like the function input) or a JSON string.
    """
    # Load the model and execute
    model = init(pa)

    predictions = []
    for instance in payload['instances']:
    # Extract parameters from input dictionary
        age = instance['age']
        height = instance['height']

    
        x = np.array([[age, height]])  # Model expects matrix
        weights = model.predict(x)
        weight = int(weights[0])  # Model produces vector

        predictions.append({'weight': weight})
    # Return back a payload with the result
    metadata = pa.endpoint_metadata()

    result = {'predictions': predictions,
        'deployedModelId': metadata.get('model_id')
    }
    return result



from baklava import images


def test_regular_version():
    result = images.tag([], 'example', '0.0.1')
    assert result.startswith('example:0.0.1')


def test_setuptools_scm_version():
    result = images.tag([], 'example', '0.0.26.dev6+g6099e47.d20200428')
    assert result.startswith('example:0.0.26.dev6.g6099e47.d20200428')




from baklava import distribution, entrypoint

# ------------------------------------------------------------------------------
# Format Dockerlines
# ------------------------------------------------------------------------------

block = """
RUN hello
RUN world
"""

block_indent = """
               RUN hello
               RUN world
               """


def test_format_dockerlines_block():
    result = distribution.format_dockerlines(block)
    assert result == block


def test_format_dockerlines_block_indent():
    result = distribution.format_dockerlines(block_indent)
    assert result == block


def test_format_dockerlines_line():
    line = "RUN hello"
    result = distribution.format_dockerlines(line)
    assert result == line


def test_format_dockerlines_single_item_list():
    items = [
        "RUN hello",
    ]
    expected = "RUN hello"
    result = distribution.format_dockerlines(items)
    assert result == expected


def test_format_dockerlines_multi_item_list():
    items = [
        "RUN hello",
        "RUN world",
    ]
    expected = '\n'.join(items)
    result = distribution.format_dockerlines(items)
    assert result == expected


def test_format_dockerlines_none():
    dockerlines = None
    expected = ""
    result = distribution.format_dockerlines(dockerlines)
    assert result == expected


def test_format_dockerlines_empty_list():
    dockerlines = []
    expected = ""
    result = distribution.format_dockerlines(dockerlines)
    assert result == expected


def test_format_dockerlines_empty_string():
    dockerlines = ""
    expected = ""
    result = distribution.format_dockerlines(dockerlines)
    assert result == expected


# ------------------------------------------------------------------------------
# Format Dockerlines
# ------------------------------------------------------------------------------

def test_format_requirements_single():
    requirements = ["hello"]
    expected = 'RUN pip install --no-cache-dir hello'
    result = distribution.format_requirements(requirements)
    assert result == expected


def test_format_requirements_multiple():
    requirements = ["hello", "world==2"]
    expected = 'RUN pip install --no-cache-dir hello world==2'
    result = distribution.format_requirements(requirements)
    assert result == expected


def test_format_requirements_empty_list():
    requirements = []
    expected = ''
    result = distribution.format_requirements(requirements)
    assert result == expected


def test_format_requirements_empty_none():
    requirements = None
    expected = ''
    result = distribution.format_requirements(requirements)
    assert result == expected


# ------------------------------------------------------------------------------
# Format Entrypoint
# ------------------------------------------------------------------------------

def test_format_entrypoints_success():
    inputs = 'alias = package.module:function'
    alias = 'sample'
    expected = 'from package.module import function as sample'

    parsed = entrypoint.parse(inputs)
    result = distribution.format_entrypoints(parsed, alias)
    assert result == expected


def test_format_entrypoints_no_entrypoint():
    parsed = (None, None, None)
    alias = 'sample'
    expected = 'sample = None'

    result = distribution.format_entrypoints(parsed, alias)
    assert result == expected


# ------------------------------------------------------------------------------
# Format Entrypoint
# ------------------------------------------------------------------------------

def test_build_parameters_success():
    result = distribution.build_parameters(
        archive="example",
        entrypoint=(None, None, None),
        requirements=[],
        python_version="5",
        dockerlines=None
    )
    assert "python_version" in result
    assert "distribution" in result
    assert "requirements" in result
    assert "dockerlines" in result
    assert "entrypoint" in result


if __name__ == '__main__':
    test_format_dockerlines_block()
    test_format_dockerlines_block_indent()
    test_format_dockerlines_line()
    test_format_dockerlines_single_item_list()
    test_format_dockerlines_multi_item_list()
    test_format_dockerlines_none()
    test_format_dockerlines_empty_list()
    test_format_dockerlines_empty_string()
    test_format_requirements_single()
    test_format_requirements_multiple()
    test_format_requirements_empty_list()
    test_format_requirements_empty_none()
    test_format_entrypoints_no_entrypoint()
    test_build_parameters_success()


"""
Environment
===========
Functions dealing with specific environment configurations.
"""
import sys

from mlbaklava import pypi


def get_environment_variables():
    """
    Use local pip installation to find environment variables that need to be
    set. This works both if using a `pip.conf` file or if using fixed
    environment variables.

    Returns:
        variables (dict[str, str]): The environment variables to set.
    """

    def fmt(x):
        return x.replace('\n', ' ').strip()

    options = pypi.load()

    return {
        'pip_index_url': fmt(options.get('index-url', "")),
        'pip_trusted_host': fmt(options.get('trusted-hosts', "")),
        'pip_extra_index_url': fmt(options.get('extra-index-url', "")),
    }


def get_python_version():
    """
    Get the python version string for the current version of python that is
    running.

    Returns:
        version (str): The major.minor version string
    """
    info = sys.version_info

    return '{major}.{minor}'.format(
        major=info.major,
        minor=info.minor,
    )



from setuptools import setup, find_packages

setup(
    name='sample',
    version='0.0.1',
    packages=find_packages(),
    entry_points={
        'baklava.train': [
            'train = sample.main:train',
        ],
        'baklava.predict': [
            'predict = sample.main:predict',
        ]
    }
)



"""
Images
======
Utility functions for building and running docker images
"""
import os
import hashlib
import tarfile
import six
import re

from distutils import log
import docker
from docker.errors import ImageNotFound, BuildError
from docker.utils.json_stream import json_stream


# ------------------------------------------------------------------------------
# Generic Operations
# ------------------------------------------------------------------------------

def check_docker():
    """
    Check that docker is actually running.

    Returns:
        status (bool): Whether or not docker is running.
    """
    try:
        import docker
        client = docker.from_env()
        client.ping()
        return True
    except:
        log.error('Docker is not running. Please open or install it.')
        return False


def build(directory, name):
    """
    Build an image using a Dockerfile at a specific path using the full name to
    tag the resulting image.

    Arguments:
        directory (str): The directory containing the Dockerfile for the
            distribution.
        name (str): The full name of the resulting image including
            the repository and tag of the image.
        buildargs (dict[str, str]): A set of arguments to pass the build
            command. These will be set as environment variables in the resulting
            image.
    """

    # Load docker client
    client = docker.from_env()

    # Attempt to find image if it exists
    try:
        client = docker.from_env()
        image = client.images.get(name)
        log.info('found image: {}'.format(name))
        return image
    except ImageNotFound:
        pass

    # Build image if it doesn't exist
    log.info('building image: {}'.format(name))
    resp = client.api.build(
        path=directory,
        tag=name,
        quiet=False,
    )

    if isinstance(resp, six.string_types):
        return client.images.get(resp)

    last_event = None
    image_id = None

    # Parse and log build stream
    for chunk in json_stream(resp):
        if 'error' in chunk:
            raise BuildError(chunk['error'], '')
        if 'stream' in chunk:
            match = re.search(
                r'(^Successfully built |sha256:)([0-9a-f]+)$',
                chunk['stream']
            )
            if match:
                image_id = match.group(2)

            formatted = chunk['stream'].rstrip()
            if formatted:
                log.info(formatted)
        last_event = chunk

    # If stream produced an image id, fetch the image
    if image_id:
        return client.images.get(image_id)

    # If no image id was produced we have an error
    raise BuildError(last_event or 'Unknown', '')


def rename(name, new):
    """
    Retags an image with a given full name to the new name.

    Arguments:
        name (str): The existing name of the image.
        new (str): The new name of the image.
    """
    client = docker.APIClient()
    client.tag(name, new)
    log.info('tagged: {}'.format(new))


def md5(artifacts):
    """
    Get the MD5 checksum for the distribution artifacts. This ignores the
    timestamp generally included in the distribution archive to be able to
    reproduce checksums solely based off of file content

    Arguments:
        artifacts (list[str]): The list of all model artifact files.

    Returns:
        checksum (str): The checksum of the tar file contents.
    """
    hash_md5 = hashlib.md5()

    for filename in artifacts:

        # Handle tar files
        if filename.endswith('tar.gz'):
            tar = tarfile.open(name=filename, mode="r:gz")
            for member in tar.getnames():
                content = tar.extractfile(member)
                if content:
                    hash_md5.update(content.read())
            continue

        # Don't hash directories
        if os.path.isdir(filename):
            continue

        # Handle all other files
        with open(filename, 'rb') as stream:
            hash_md5.update(stream.read())

    return hash_md5.hexdigest()


def tag(artifacts, name, version):
    """
    Creates a tag with the following naming convention:

        name:version-checksum

    For example:

        example:0.0.1-af873cebe8
        example:0.0.1-af873cebe8

    Note: The purpose of including an artifact checksum in the tag is so that
    an identical image can be looked up immediately without rebuilding the
    image. By default `.tar.gz` files include a timestamp of when the archive
    was created which means `sdist` files can never be cached unless manually
    looking at the contents (i.e. the `images.md5` routine)

    Arguments:
        artifacts (list[str]): The locations of the distribution artifacts
        name (str): The name of the distribution
        version (str): The version of the distribution

    Returns:
        str: The tag string
    """
    checksum = md5(artifacts)
    docker_tag = '{}-{}'.format(version, checksum)

    # Here we remove random punctuation to be compliant with docker tagging.
    # This scenario will occur when using `setuptools-scm` in development
    docker_tag = re.sub(r"[^\w.-]", '.', docker_tag)

    return '{}:{}'.format(name, docker_tag)



from setuptools import setup, find_packages
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='mlbaklava',
    long_description=long_description,
    long_description_content_type='text/markdown',
    use_scm_version={
        'local_scheme': 'no-local-version',
        'write_to': 'mlbaklava/__version.py',
        'write_to_template': '__version__ = \'{version}\''
    },
    maintainer='Intuit ML Platform',
    maintainer_email='baklava-maintainers@intuit.com',
    url='https://github.com/intuit/baklava',
    packages=find_packages(),
    include_package_data=True,
    setup_requires=[
        'setuptools_scm',
    ],
    install_requires=[
        'docker>=2.0.0',   # Earliest with compatible `docker.from_env` API
        'appdirs>=1.4.0',  # First version compatible with current pip paths
        'six>=1.16.0'
    ],
    entry_points={
        'distutils.commands': [
            'process = mlbaklava.commands:Process',
            'train = mlbaklava.commands:Train',
            'batch = mlbaklava.commands:Batch',
            'deploy = mlbaklava.commands:Deploy',
        ],
        'distutils.setup_keywords': [
            'python_version = mlbaklava.commands:passthrough',
            'dockerlines = mlbaklava.commands:passthrough',
        ],
    }
)



from random_word import RandomWords
import boto3
client = boto3.Session().client('sagemaker')

words = RandomWords().get_random_words()
response = client.create_training_job(
    TrainingJobName=f'mlctl-{words[0]}-{words[1]}',
    # HyperParameters={
    #     'string': 'string'
    # },
    AlgorithmSpecification= {
      "TrainingImage": '436885317446.dkr.ecr.us-east-1.amazonaws.com/mlctl-test:train-image',
      "TrainingInputMode": "File"
    },
    RoleArn='arn:aws:iam::436885317446:role/sm-execution',
    InputDataConfig=[
        {
            'ChannelName': 'training',
            'DataSource': {
                'S3DataSource': {
                    'S3DataType': 'S3Prefix',
                    'S3Uri': 's3://mlctltest/example1_data/',
                    'S3DataDistributionType': 'FullyReplicated',
                }
            },
            'ContentType': 'text/csv',
            # 'CompressionType': 'None'|'Gzip',
            # 'RecordWrapperType': 'None'|'RecordIO',
            'InputMode': 'File',
            # 'ShuffleConfig': {
            #     'Seed': 123
            # }
        },
    ],
    OutputDataConfig={
        'S3OutputPath': 's3://mlctltest/example1_output/'
    },
    ResourceConfig={
        'InstanceType': 'ml.m5.large',
        'InstanceCount': 1,
        'VolumeSizeInGB': 5,
    },
    StoppingCondition={
        'MaxRuntimeInSeconds': 1800
    },
    Tags=[
        {
            'Key': 'mlctl',
            'Value': 'local'
        },
    ],
    # EnableManagedSpotTraining=True|False,
    # CheckpointConfig={
    #     'S3Uri': 'string',
    #     'LocalPath': 'string'
    # },
    # TensorBoardOutputConfig={
    #     'LocalPath': 'string',
    #     'S3OutputPath': 'string'
    # },
    # ExperimentConfig={
    #     'ExperimentName': 'mlctl_test',
    #     'TrialName': 'v1',
    #     'TrialComponentDisplayName': 'training_step'
    # },
    # ProfilerConfig={
    #     'S3OutputPath': 'string',
    #     'ProfilingIntervalInMilliseconds': 123,
    #     'ProfilingParameters': {
    #         'string': 'string'
    #     }
    # },
    # ProfilerRuleConfigurations=[
    #     {
    #         'RuleConfigurationName': 'string',
    #         'LocalPath': 'string',
    #         'S3OutputPath': 'string',
    #         'RuleEvaluatorImage': 'string',
    #         'InstanceType': 'ml.t3.medium'|'ml.t3.large'|'ml.t3.xlarge'|'ml.t3.2xlarge'|'ml.m4.xlarge'|'ml.m4.2xlarge'|'ml.m4.4xlarge'|'ml.m4.10xlarge'|'ml.m4.16xlarge'|'ml.c4.xlarge'|'ml.c4.2xlarge'|'ml.c4.4xlarge'|'ml.c4.8xlarge'|'ml.p2.xlarge'|'ml.p2.8xlarge'|'ml.p2.16xlarge'|'ml.p3.2xlarge'|'ml.p3.8xlarge'|'ml.p3.16xlarge'|'ml.c5.xlarge'|'ml.c5.2xlarge'|'ml.c5.4xlarge'|'ml.c5.9xlarge'|'ml.c5.18xlarge'|'ml.m5.large'|'ml.m5.xlarge'|'ml.m5.2xlarge'|'ml.m5.4xlarge'|'ml.m5.12xlarge'|'ml.m5.24xlarge'|'ml.r5.large'|'ml.r5.xlarge'|'ml.r5.2xlarge'|'ml.r5.4xlarge'|'ml.r5.8xlarge'|'ml.r5.12xlarge'|'ml.r5.16xlarge'|'ml.r5.24xlarge'|'ml.g4dn.xlarge'|'ml.g4dn.2xlarge'|'ml.g4dn.4xlarge'|'ml.g4dn.8xlarge'|'ml.g4dn.12xlarge'|'ml.g4dn.16xlarge',
    #         'VolumeSizeInGB': 123,
    #         'RuleParameters': {
    #             'string': 'string'
    #         }
    #     },
    # ],
    Environment={
        'sriracha_provider': 'awssagemaker'
    },
    # RetryStrategy={
    #     'MaximumRetryAttempts': 123
    # }
)

print(response)


from setuptools import setup, find_packages


def lines(filename):
    """
    Read each line of a file into a list

    Args:
        filename (str): The name of the file to read
    Returns:
        lines (list[str]): The lines within the file
    """
    with open(filename) as stream:
        return stream.read().splitlines()


def requirements():
    return lines('requirements.txt')


def dockerlines():
    return lines('dockerlines.txt')


setup(
    name='non_python_files',
    version='0.0.1',
    packages=find_packages(),
    include_package_data=True,

    # Specify requirements using text file
    install_requires=requirements(),

    # Specify baklava information
    python_version='2.7.15',
    dockerlines=dockerlines(),
    entry_points={
        'baklava.train': [
            'train = non_python_files.train:main',
        ],
        'baklava.initialize': [
            'init = non_python_files.predict:load_model',
        ],
        'baklava.predict': [
            'pred = non_python_files.predict:main',
        ]
    }
)



import pytest
from baklava import entrypoint


entries = {
    'baklava.train': [
        'alias1 = package1.module1:function1',
        'alias2 = package2.module2:function2',
        'alias3 = package3.module3:function3',
    ],
    'baklava.predict': [
        'alias = package.module:function',
    ],
    'console_scripts': [
        'alias = package.module:function',
    ],
}


train = 'baklava.train'
predict = 'baklava.predict'


@pytest.mark.xfail(raises=AssertionError)
def test_entrypoint_get_ambiguous():
    """Error when ambiguous reference"""
    entrypoint.get(entries, train)


@pytest.mark.xfail(raises=AssertionError)
def test_entrypoint_get_empty():
    """Error when empty entries are provided"""
    entrypoint.get({}, train)


@pytest.mark.xfail(raises=AssertionError)
def test_entrypoint_get_none():
    """Error when no entry provided"""
    entrypoint.get(None, train)


@pytest.mark.xfail(raises=AssertionError)
def test_entrypoint_get_type_error():
    """Error when no entry provided"""
    entrypoint.get({'baklava.train': 'error'}, train)


@pytest.mark.xfail(raises=AssertionError)
def test_entrypoint_get_empty_list():
    """Error when no entry provided"""
    entrypoint.get({'baklava.train': []}, train)


def test_entrypoint_get_single():
    """Single unambiguous entrypoint"""
    name, package, func = entrypoint.get(entries, predict)
    assert name == 'alias'
    assert package == 'package.module'
    assert func == 'function'


def test_entrypoint_get_multiple():
    """Multiple entrypoint with specifier"""
    name, package, func = entrypoint.get(entries, train, 'alias2')
    assert name == 'alias2'
    assert package == 'package2.module2'
    assert func == 'function2'


def test_parse():
    string = ' alias = package.module:function '
    name, package, func = entrypoint.parse(string)
    assert name == 'alias'
    assert package == 'package.module'
    assert func == 'function'


def test_parse_no_space():
    string = 'alias=package.module:function'
    name, package, func = entrypoint.parse(string)
    assert name == 'alias'
    assert package == 'package.module'
    assert func == 'function'


@pytest.mark.xfail(raises=AssertionError)
def test_parse_comma_error():
    string = (
        'alias = package.module:function'
        'alias = package.module:function'
    )
    entrypoint.parse(string)


if __name__ == '__main__':
    pytest.main([__file__])
